/***************************************************************
* InspectorApp -------------------------------------------------
*  Extending InspectorApp to add XPath-handling code.
****************************************************************/

dump("chrome://xpath/content/xpath.js loaded\n");

/*
DOMViewer.subject is the root of the DOM document.
DOMViewer is accessable via this.mDocPanel.viewer.
DOMViewer.mDOMView provides access to the 
  @mozilla.org/inspector/dom-view;1 component, or the inIDOMView 
  and nsITreeView interfaces.
*/

window.addEventListener("load", XPath_initialize, false);

const kArrayCID = "@mozilla.org/supports-array;1";

function convert_nsISupportsArray(obj) {
  if (obj.ElementAt) {
    consoleDump("obj is something");
    var val = new Array(obj.count);
    var prop = null;
    var i = 0;
    for (prop = obj.ElementAt(i); i < obj.count(); i++) {
      val[i] = prop;
    }
    return val;
  } else {
    consoleDump("obj is nothing! waaa!!!");
    return new Array(0);
  }
}
  

function XPath_initialize() {
  //if (!inspector) InspectorApp_initialize();
  inspector.initXPath();
}

InspectorApp.prototype.initXPath =
  function()
  {
    this.mPendingXPath = null;
    this.mPendingNoSaveXPath = null;
    this.xProps = null;
    this.xTreeView = this.xDOMView = null;
    this.xTree = null;
    this.xpathDocument = null;
    this.xView = null;
  };

InspectorApp.prototype.__defineGetter__("xpathStatus", 
  function(value)
  {
    var xpathbar = document.getElementById("tfXPathBar");
    return xpathbar.getAttribute("status");
  }
);

InspectorApp.prototype.__defineSetter__("xpathStatus", 
  function(value)
  {
    var xpathbar = document.getElementById("tfXPathBar");
    if (value == "") {
      xpathbar.removeAttribute("status");
    } else {
      xpathbar.setAttribute("status", value);
    }
    return value;
  }
);

//See gotoTypedURL()
InspectorApp.prototype.gotoTypedXPath =
  function()
  {
    var xpath = document.getElementById("tfXPathBar").value;
    dump("InspectorApp.gotoTypedXPath("+xpath+")\n");
    this.gotoXPath(xpath);
  };

//See gotoURL()
InspectorApp.prototype.gotoXPath =
  function(aXPath, aNoSaveHistory)
  {
    this.mPendingXPath = aXPath;
    this.mPendingNoSaveXPath = aNoSaveHistory;
    this.browseToXPath(aXPath);
    //this.setBrowser(true, true);
  };

//See browseToURL()
InspectorApp.prototype.browseToXPath =
  function(aPath)
  {
    dump("InspectorApp.browseToXPath("+aPath+")\n");
    /*var col = 'colNodeName';
    var row = 0;
    var props = XPCU.createInstance(kArrayCID, 'nsISupportsArray');*/
    
    if (!this.xView) {
      this.xView = this.mDocPanel.viewer;
    }
    if (!this.xTreeView) {
      this.xTreeView = this.xDOMView = this.xView.mDOMView;
    }
    if (!this.xTree) {
      this.xTree = this.xView.mDOMTree;
    }
    
    this.xDocument = this.document;
    
    try {
      var result = this.xDocument.evaluate(aPath, 
          this.xDocument, this.xLookupNamespace,
          XPathResult.UNORDERED_NODE_ITERATOR_TYPE, null);
    } catch(err) {
      //XPathException.INVALID_EXPRESSION_ERR
      this.xpathStatus = "syntaxerror";
      consoleDump("browseToXPath() 1: "+err);
      return;
    }
    
    var res = null;
    var has_one = false;
    this.xView.xClearSelection();
    try {
      while (res = result.iterateNext()) {
        has_one = true;
        this.xView.xSelectNode( res );
        consoleDump(res.nodeName);
      }
    } catch(err) {
      consoleDump("browseToXPath() 2: "+err);
//      return;
    }
    this.xpathStatus = has_one ? "": "notfound";
  };
  
//See get locationText()
InspectorApp.prototype.__defineGetter__("xpathText", 
  function() {
    return document.getElementById("tfXPathBar").value;
  }
);
//See set locationText()
InspectorApp.prototype.__defineSetter__("xpathText", 
  function(aText) {
    return (document.getElementById("tfXPathBar").value = aText);
  }
);

//See documentLoaded()
InspectorApp.prototype.xpathLoaded =
  function()
  {
    this.setTargetWindow(_content);

    // No reference object to use.
    var xpath = this.xpathText;
    
    // put the XPath into the xpathbar
    //this.xpathText = xpath; Redundent

    // add url to the history, unless explicity told not to
    if (!this.mPendingNoSaveXPath)
      this.addToHistoryXPath(xpath);

    this.mPendingXPath = null;
    this.mPendingNoSaveXPath = null;
  };

//See addToHistory()
InspectorApp.prototype.addToHistoryXPath =
  function(aPath)
  {
  };

InspectorApp.prototype.xLookupNamespace = 
  function (prefix) {
    return this.xDocument.lookupNamespaceURI(prefix);
  };

function xpathBarChange() {
  inspector.xpathStatus = "";
}
